import configparser
import os
import re
import argparse
import sys

from simple_term_menu import TerminalMenu
from cryptography.fernet import Fernet
from .box import ConfUtils, PasswordManager


parser = argparse.ArgumentParser(description='Mini Password is a command-line password manager.')
group = parser.add_mutually_exclusive_group()
group.add_argument('-a', '--add', action='store_true', help='add a new password')
group.add_argument('-d', '--delete', action='store_true', help='delete a password')
group.add_argument('-u', '--update', action='store_true', help='update a password')
group.add_argument('-l', '--list-file', action='store_true', help='list database file and configuration file paths')
group.add_argument('-b', '--backup', action='store_true', help='backup the database file')
group.add_argument('-p', '--upload', action='store_true', help='upload the database file to a cloud service')
group.add_argument('-r', '--restore', action='store_true', help='restore the database file from cloud service')
group.add_argument('-api', '--api', action='store_true', help='set cloud API url and token')
group.add_argument('-dd', '--destroy', action='store_true', help='destroy the database, all data will be lost!')
args = parser.parse_args()


class CommandHandler:

    def __init__(self):
        if len(sys.argv) > 2:
            print('Only one argument is allowed!')

        self.conf_utils = ConfUtils()
        self.run_trigger = True
        self.add_arg = args.add
        self.delete_arg = args.delete
        self.update_arg = args.update
        self.list_files_arg = args.list_file
        self.destroy_arg = args.destroy
        self.backup_arg = args.backup
        self.upload_arg = args.upload
        self.restore_arg = args.restore
        self.api_arg = args.api


        try:
            self.conf_utils.get('db', 'database_file')
            self.conf_utils.get('common', 'aes_key')
        except (configparser.NoSectionError, configparser.NoOptionError):

            self.run_trigger = False
            try:
                res = input(f'Enter the database file path ({self.conf_utils.data_path}/{self.conf_utils.DB_FILENAME}): ')
                if res == '':
                    self.conf_utils.create_database_file()
                else:
                    self.conf_utils.create_database_file(res)

                df = self.conf_utils.get('db', 'database_file')
                print(f'Database file created: {df}')

                aes_key = Fernet.generate_key()
                aes_key_string = aes_key.decode('utf-8')
                self.conf_utils.set('common', 'aes_key', aes_key_string)
                print(f'AES key is generated, please keep it in a safe place: {aes_key_string}')

                self.conf_utils.config.read(self.conf_utils.config_file)
            except KeyboardInterrupt:
                print('Interrupted by user!')
                return

        self.password_manager = PasswordManager(conf_utils=self.conf_utils)

    def run(self):

        if self.add_arg:
            self.add_password()
            return

        if self.delete_arg:
            self.delete_password()
            return

        if self.update_arg:
            self.update_password()
            return

        if self.list_files_arg:
            self.list_files()
            return

        if self.backup_arg:
            self.backup_db()
            return

        if self.upload_arg:
            self.upload()
            return

        if self.restore_arg:
            self.restore()
            return

        if self.api_arg:
            self.set_api_token()
            return

        if self.destroy_arg:
            confirm = input('Are you sure you want to destroy the database file? If you have backups, ensure that you also have the secret key to restore (Y/n):')
            if confirm == 'Y':
                self.password_manager.destroy_db()
                print('Database destroyed!')
            return

        if not self.run_trigger:
            return

        os.system('clear')
        try:
            query_string = input('Enter a query string: ')
            passwords = self.password_manager.get_password(query_string)
            if len(passwords) > 0:
                if len(passwords) == 1:
                    self.show_password(passwords[0])
                else:
                    menu_items = []
                    for password in passwords:
                        item = f'#{password[0]} \| {password[1]} \| {password[5]} \| {password[4]}'
                        if len(item) > 40:
                            item = item[:40] + '...'
                        menu_items.append(item)
                    menu = TerminalMenu(menu_items)
                    menu_entry_index = menu.show()
                    selected_password = passwords[menu_entry_index]
                    self.show_password(selected_password)
            else:
                print('No password found!')

        except KeyboardInterrupt:
            print('Interrupted by user!')
            return

    def add_password(self):
        try:
            os.system('clear')
            name = input('Enter the name: ')

            if name == '':
                print('Name is required!')
                return

            password_obj = self.password_manager.get_password_by_name(name)
            if password_obj:
                print(f'Password already exists! (name is {name})')
                return

            login_name = input('Enter the login name: ')
            if login_name == '':
                print('Login name is required!')
                return

            password = input('Enter the password: ')
            if password == '':
                print('Password is required!')
                return

            memo = input('Enter the memo: ')
            url = input('Enter the url: ')
            if url != '' and not self.is_valid_url(url):
                while True:
                    url = input('Invalid url! Please enter a valid url or leave it blank: ')
                    if self.is_valid_url(url) or url == '':
                        break

            self.password_manager.add_password(name, login_name, password, memo, url)
            print('Password added!')
        except KeyboardInterrupt:
            print('Interrupted by user!')
            return

    def delete_password(self):
        try:
            password_id = input('Enter password id: ')
            confirm = input(f'Are you sure you want to delete password id {password_id}? (y/n): ')
            if confirm.lower() != 'y':
                return
            self.password_manager.delete_password(password_id)
            print('Password deleted!')
        except KeyboardInterrupt:
            print('Interrupted by user!')
            return

    def update_password(self):
        try:
            password_id = input('Enter password id: ')
            if password_id == '':
                print('Password id is required!')
                return

            password_obj = self.password_manager.get_password_by_id(password_id)
            name = input(f'Enter the name ({password_obj[1]}): ')
            if name == '':
                name = password_obj[1]
            else:
                if self.password_manager.get_password_by_name(name) and name != password_obj[1]:
                    print(f'Password already exists! (name is {name})')
                    return
            original_login_name = self.password_manager.aes_decrypt(password_obj[2])
            login_name = input(f'Enter the login name ({original_login_name}): ')
            if login_name == '':
                login_name = original_login_name

            password = input('Enter the password: ')
            if password == '':
                password = self.password_manager.aes_decrypt(password_obj[3])
            memo = input(f'\n\n({password_obj[4]})\n\nEnter the memo: ')
            if memo == '':
                memo = password_obj[4]
            url = input(f'Enter the url ({password_obj[5]}): ')
            if url == '':
                url = password_obj[5]

            if url != '' and not self.is_valid_url(url):
                while True:
                    url = input('Invalid url! Please enter a valid url: ')
                    if self.is_valid_url(url):
                        break

            self.password_manager.update_password(password_id, name, login_name, password, memo, url)
            print('Password updated!')
        except KeyboardInterrupt:
            print('Interrupted by user!')
            return

    def list_files(self):
        print(f'database file: {self.conf_utils.get("db", "database_file")}')
        print(f'configuration file: {self.conf_utils.config_file}')

    def backup_db(self):
        backup_file = input('Please enter the backup file path and file name (the default location is in the same folder as the database file): ')
        if backup_file == '':
            backup_file = self.password_manager.backup_db()
        else:
            backup_file = self.password_manager.backup_db(backup_file)
        print(f'Database file backed up to {backup_file}')

    def upload(self):
        if self.check_api_token():
            result = self.password_manager.upload_db()
            if result.status_code == 200:
                # todo: need to check the server response a valid message
                print(f'Database file uploaded to cloud! {result.text}')
                print(f'API url: {self.conf_utils.get("common", "cloud_api")}')
            else:
                print(f'Upload failed! {result.status_code} {result.text}')
                print(f'API url: {self.conf_utils.get("common", "cloud_api")}')

    def restore(self):
        if self.check_api_token():
            url = self.conf_utils.get('common', 'cloud_api')
            confirm = input(f'Are you sure you want to restore the database file from the cloud service? \nPlease ensure that the API URL provided is valid: {url} (Y/n): ')
            if confirm == 'Y':
                result = self.password_manager.restore_db()
                if result.status_code == 200:
                    print(f'Database file restored!')
                    print(f'API url: {self.conf_utils.get("common", "cloud_api")}')
                    print(f'Database file path: {self.conf_utils.get("db", "database_file")}')
                else:
                    print(f'Restored failed! {result.status_code} {result.text}')
                    print(f'API url: {self.conf_utils.get("common", "cloud_api")}')

    def show_password(self, selected_password):
        print('+++++++++++++++++++++++++++++++')
        password = self.password_manager.aes_decrypt(selected_password[3])
        login_name = self.password_manager.aes_decrypt(selected_password[2])
        print(f'\nlogin name: {login_name}')
        print(f'password: {password} \n\n')
        print(f'id: {selected_password[0]}')
        print(f'name: {selected_password[1]}')
        print(f'url: {selected_password[5]}')
        print(f'memo: {selected_password[4]}\n')
        print('+++++++++++++++++++++++++++++++')

    def check_api_token(self):

        try:
            url = self.conf_utils.get('common', 'cloud_api')
            token = self.conf_utils.get('common', 'cloud_token')
            return True
        except (configparser.NoOptionError, configparser.NoSectionError):
            return self.set_api_token()

    def set_api_token(self):
        try:
            url = input('Enter the cloud API url: ')
            if url == '':
                print('Cloud API url is required!')
                return False

            if not self.is_valid_url(url):
                print('Invalid url!')
                return False

            token = input('Enter the cloud API token: ')
            if token == '':
                print('Cloud API token is required!')
                return False

            self.conf_utils.set('common', 'cloud_api', url)
            self.conf_utils.set('common', 'cloud_token', token)
        except KeyboardInterrupt:
            print('Interrupted by user!')
            return False

        return True

    @staticmethod
    def is_valid_url(url):
        regex = re.compile(
            r'^(?:http|ftp)s?://'  # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
            r'localhost|'  # localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
            r'(?::\d+)?'  # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)
        return re.match(regex, url) is not None




import os
import random
import unittest
from minipassword.box import PasswordManager


class TestPasswordManager(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestPasswordManager, self).__init__(*args, **kwargs)
        self.password_manager = PasswordManager()

    def test_add_password(self):
        result = self.password_manager.add_password(f'mypassword{random.randint(1, 1000)}', 'mypassword', 'mypassword')
        print('Add Password done!')
        self.assertIsNone(result)

    def test_get_password(self):
        result = self.password_manager.get_password('test')
        print('Get Password:')
        print(result)
        self.assertIsNotNone(result)

    def test_get_all_password(self):
        result = self.password_manager.get_all_passwords()
        print('Get All Passwords:')
        print(result)
        self.assertIsNotNone(result)

    def test_update_password(self):
        try:
            self.password_manager.update_password(1, 'updatedpassword', 'updatedpassword', 'updatedpassword')
            result = self.password_manager.get_password('updatedpassword')
            print('Update Password done!')
            print(result)
            self.assertTrue(True)
        except Exception as e:
            print(e)
            self.assertTrue(False)

    def test_delete_password(self):
        try:
            self.password_manager.delete_password(1)
            result = self.password_manager.get_all_passwords()
            print('Delete Password done!')
            print(result)
            self.assertTrue(True)
        except Exception as e:
            print(e)
            self.assertTrue(False)


if __name__ == '__main__':
    unittest.main()



import os
import requests
import glob
import datetime
import configparser
import sqlite3
import shutil
from cryptography.fernet import Fernet


class ConfUtils:
    DB_FILENAME = 'minipassword.db'

    def __init__(self):
        self.data_path = f'{os.path.expanduser("~")}/.minipassword'
        if not os.path.exists(self.data_path):
            os.makedirs(self.data_path)
        self.config = configparser.ConfigParser()
        self.config_file = f'{self.data_path}/config.ini'
        self.database_file = f'{self.data_path}/{self.DB_FILENAME}'

        if not os.path.exists(self.config_file):
            with open(self.config_file, 'w') as f:
                self.config.write(f)

            self.config.add_section('common')
            self.config.add_section('db')

        self.config.read(self.config_file)

    def get(self, section: str, key: str):
        return self.config.get(section, key)

    def set(self, section: str, key: str, value: str):
        self.config.set(section, key, value)
        with open(self.config_file, 'w') as f:
            self.config.write(f)

        self.config.read(self.config_file)

    def create_database_file(self, database_path=None):
        if database_path is not None:
            self.database_file = f'{database_path}/{self.DB_FILENAME}'

        self.set('db', 'database_file', self.database_file)
        conn = sqlite3.connect(self.database_file)

        # Create table
        cursor = conn.cursor()
        cursor.execute('''
            CREATE TABLE IF NOT EXISTS passwords (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                name VARCHAR(200) NOT NULL UNIQUE,
                login_name TEXT NOT NULL,
                password TEXT NOT NULL,
                memo TEXT NULL,
                url VARCHAR(255) NULL
            )
        ''')

        conn.close()

        self.config.read(self.config_file)


class PasswordManager:

    def __init__(self, conf_utils=None):
        if conf_utils is None:
            self.conf_utils = ConfUtils()
        else:
            self.conf_utils = conf_utils

        self.database_file = self.conf_utils.get('db', 'database_file')
        self.aes_key = self.conf_utils.get('common', 'aes_key')

    def add_password(self, name, login_name, password, memo=None, url=None):
        conn = sqlite3.connect(self.database_file)
        password = self.aes_encrypt(password)
        login_name = self.aes_encrypt(login_name)
        cursor = conn.cursor()
        cursor.execute('''
            INSERT INTO passwords (name, login_name, password, memo, url)
            VALUES (?, ?, ?, ?, ?)
        ''', (name, login_name, password, memo, url))
        conn.commit()
        conn.close()

    def get_password(self, query_string):
        conn = sqlite3.connect(self.database_file)
        cursor = conn.cursor()
        cursor.execute('''
            SELECT * FROM passwords WHERE name LIKE ? OR url LIKE ? OR MEMO LIKE ?
        ''', (f'%{query_string}%', f'%{query_string}%', f'%{query_string}%'))
        result = cursor.fetchall()
        conn.close()
        return result

    def get_password_by_id(self, password_id):
        conn = sqlite3.connect(self.database_file)
        cursor = conn.cursor()
        cursor.execute('''
            SELECT * FROM passwords WHERE id=?
        ''', (password_id,))
        result = cursor.fetchone()
        conn.close()
        return result

    def get_password_by_name(self, name):
        conn = sqlite3.connect(self.database_file)
        cursor = conn.cursor()
        cursor.execute('''
            SELECT * FROM passwords WHERE name=?
        ''', (name,))
        result = cursor.fetchone()
        conn.close()
        return result

    def get_all_passwords(self):
        conn = sqlite3.connect(self.database_file)
        cursor = conn.cursor()
        cursor.execute('''
            SELECT * FROM passwords
        ''')
        result = cursor.fetchall()
        conn.close()
        return result

    def update_password(self, password_id, name, login_name, password, memo=None, url=None):
        login_name = self.aes_encrypt(login_name)
        password = self.aes_encrypt(password)
        conn = sqlite3.connect(self.database_file)
        cursor = conn.cursor()
        cursor.execute('''
            UPDATE passwords
            SET name=?, login_name=?, password=?, memo=?, url=?
            WHERE id=?
        ''', (name, login_name, password, memo, url, password_id))
        conn.commit()
        conn.close()

    def delete_password(self, password_id):
        conn = sqlite3.connect(self.database_file)
        cursor = conn.cursor()
        cursor.execute('''
            DELETE FROM passwords WHERE id=?
        ''', (password_id,))
        conn.commit()
        conn.close()

    def backup_db(self, backup_file=None):
        if backup_file is None:
            now = datetime.datetime.now()
            date_str = now.strftime('%Y%m%d%H%M%S')
            name, extension = os.path.splitext(self.database_file)
            backup_file = f'{name}_bak_{date_str}{extension}'

        shutil.copyfile(self.database_file, backup_file)

        return backup_file

    def upload_db(self):
        self.conf_utils.config.read(self.conf_utils.config_file)

        url = self.conf_utils.get('common', 'cloud_api')
        token = self.conf_utils.get('common', 'cloud_token')
        with open(self.database_file, 'rb') as file:
            headers = {
                'Content-Type': 'application/octet-stream',
                'Authorization': f'Bearer {token}'
            }

            response = requests.post(url, headers=headers, data=file)

            return response

    def restore_db(self):
        url = self.conf_utils.get('common', 'cloud_api')
        token = self.conf_utils.get('common', 'cloud_token')
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {token}'
        }
        # need to check the content of the response if it is a valid database file?
        response = requests.post(url, headers=headers)
        if response.status_code == 200:
            # before writing the file, need to check if the response content is a valid database file
            with open(self.database_file, 'wb') as file:
                file.write(response.content)
        return response

    def destroy_db(self):
        if os.path.exists(self.database_file):
            os.remove(self.database_file)
        files = glob.glob(f'{self.conf_utils.data_path}/*')
        for f in files:
            if os.path.isfile(f):
                os.remove(f)
        if os.path.exists(self.conf_utils.config_file):
            os.remove(self.conf_utils.config_file)

    def aes_encrypt(self, data):
        f = Fernet(self.aes_key.encode())
        return f.encrypt(data.encode()).decode()

    def aes_decrypt(self, encrypt_data):
        f = Fernet(self.aes_key.encode())
        return f.decrypt(encrypt_data.encode()).decode()








import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="minipassword",
    version="1.0.2",
    author="Laonan",
    author_email="hello@laonan.net",
    description="Mini Password is a command-line password manager.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/laonan/minipassword",  # github url
    project_urls={
        "Bug Tracker": "https://github.com/laonan/minipassword/issues",
    },
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    zip_safe=True,
    install_requires=[
        'simple-term-menu>=1.6.4',
        'cryptography>=42.0.7',
        'requests>=2.31.0',
    ],
    python_requires=">=3.6",
)



import os
import unittest
from minipassword.box import ConfUtils


class TestConfig(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestConfig, self).__init__(*args, **kwargs)

    def test_create_database_file(self):
        util = ConfUtils()
        util.create_database_file()
        if os.path.isfile(util.config_file) and os.path.isfile(util.get('db', 'database_file')):
            self.assertTrue(True)
        else:
            self.assertTrue(False)


if __name__ == '__main__':
    unittest.main()
